{
  SysMenu 1.0
  Sergey Kolchin, 19.03.98

  The TKolSystemMenu allows to install any Popup menu to form's system
  menu. All item's properties and events are handled
}
unit SysMenu;

interface                                         
uses Classes, SysUtils, Windows, Menus, Messages, Forms, Dialogs, Controls;

type                                    
  TKolSystemMenu = class(TComponent)
  private
		FPopupMenu: TPopupMenu;
		FPosition: integer;
    fOldOnMessage: TMessageEvent;
		procedure SetPopupMenu (Value: TPopupMenu);
		procedure SetPosition (Value: integer);
		procedure UpdateMenu;
    procedure AppendTo(Item: TMenuItem; Menu: HMENU; nPos: integer);
    procedure AppMessage (var Msg: TMsg; var Handled: Boolean);
		procedure Loaded; override;                                    
	public
		constructor Create (AnOwner: TComponent); override;
		destructor Destroy; override;
		function HandleSysMenu (var Msg: TMsg): boolean;
      // Call this function from own handler of Application.OnMessage
	published
		property PopupMenu: TPopupMenu read FPopupMenu write SetPopupMenu;
		property Position: integer read FPosition write SetPosition default -1;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents ('Freeware', [TKolSystemMenu]);
end;

//***************************************************
// Taken from Delphi's Menus unit, should be changed accordingly
//***************************************************
procedure TKolSystemMenu.AppendTo;
const
  IBreaks: array[TMenuBreak] of Longint = (MFT_STRING, MFT_MENUBREAK, MFT_MENUBARBREAK);
  IChecks: array[Boolean] of Longint = (MFS_UNCHECKED, MFS_CHECKED);
  IDefaults: array[Boolean] of Longint = (0, MFS_DEFAULT);
  IEnables: array[Boolean] of Longint = (MFS_DISABLED or MFS_GRAYED, MFS_ENABLED);
  IRadios: array[Boolean] of Longint = (MFT_STRING, MFT_RADIOCHECK);
  ISeparators: array[Boolean] of Longint = (MFT_STRING, MFT_SEPARATOR);
var
  MenuItemInfo: TMenuItemInfo;
  aCaption: string;
  NewFlags: Integer;
begin
  if Item.Visible then with Item do begin
    aCaption := Caption;
    if Count > 0 then MenuItemInfo.hSubMenu := Handle
    else if (ShortCut <> scNone) and ((Parent = nil) or
      (Parent.Parent <> nil) or not (Parent.Owner is TMainMenu)) then
      aCaption := aCaption + #9 + ShortCutToText(ShortCut);
    if Lo(GetVersion) >= 4 then begin
      MenuItemInfo.cbSize := SizeOf(TMenuItemInfo);
      MenuItemInfo.fMask := MIIM_CHECKMARKS or MIIM_DATA or MIIM_ID or
        MIIM_STATE or MIIM_SUBMENU or MIIM_TYPE;
      MenuItemInfo.fType := IRadios[RadioItem] or IBreaks[Break] or
        ISeparators[Caption = '-'];
      MenuItemInfo.fState := IChecks[Checked] or IEnables[Enabled]
        or IDefaults[Default];
      MenuItemInfo.wID := Command;
      MenuItemInfo.hSubMenu := 0;
      MenuItemInfo.hbmpChecked := 0;
      MenuItemInfo.hbmpUnchecked := 0;
      MenuItemInfo.dwTypeData := PChar(aCaption);
      if Count > 0 then MenuITemInfo.hSubMenu := Handle;
      InsertMenuItem(Menu, nPos, True, MenuItemInfo);
    end
    else begin
      NewFlags := IBreaks[Break] or IChecks[Checked] or IEnables[Enabled] or
        ISeparators[Caption = '-'] or MF_BYPOSITION;
      if Count > 0 then
        InsertMenu(Menu, nPos, MF_POPUP or NewFlags, Handle,
          PChar(aCaption))
      else
        InsertMenu(Menu, nPos, NewFlags, Command, PChar(aCaption));
    end;
  end;
end;

//***************************************************
procedure TKolSystemMenu.SetPopupMenu;
begin
	if FPopupMenu <> Value then begin
		FPopupMenu := Value;
    UpdateMenu;
	end;
end;

//***************************************************
procedure TKolSystemMenu.UpdateMenu;
var
  mHandle: THandle;
  i, n: integer;
begin
  if (csDesigning in ComponentState) then exit;
  GetSystemMenu ((Owner as TForm).Handle, False);
  mHandle := GetSystemMenu (TForm(Owner).Handle, False);
  if not assigned(PopupMenu) then exit;

  for i := 0 to PopupMenu.Items.Count-1 do begin
    if Position = -1 then n := -1 else n := Position + i;
    AppendTo (PopupMenu.Items[i], mHandle, n);
  end;
end;

//***************************************************
procedure TKolSystemMenu.SetPosition;
begin
	if FPosition <> Value then begin
		FPosition := Value;
    UpdateMenu;
	end;
end;

//***************************************************
constructor TKolSystemMenu.Create;
begin
	inherited Create (AnOwner);
  FPosition := -1;
end;

//***************************************************
function TKolSystemMenu.HandleSysMenu;

function DoMenu (Item: TMenuItem): boolean;
var
  i: integer;
begin
  Result := False;
  for i := 0 to Item.Count-1 do begin
    if Item.Items[i].Command = LoWord(Msg.wParam) then begin
      Item.Items[i].Click;
      Result := True;
      break;
    end
    else DoMenu (Item.Items[i]);
  end;
end;

begin
  Result := False;
	inherited;
  if not assigned(PopupMenu) then exit;
  DoMenu (PopupMenu.Items);
end;

//***************************************************
procedure TKolSystemMenu.AppMessage (var Msg: TMsg; var Handled: Boolean);
begin
  if assigned (fOldOnMessage) then fOldOnMessage(Msg, Handled);
  if (not Handled) and (Msg.message = WM_SYSCOMMAND) then begin
    Handled := HandleSysMenu (Msg);
  end;
end;

//***************************************************
destructor TKolSystemMenu.Destroy;
begin
  Application.OnMessage := fOldOnMessage;
	inherited Destroy;
end;

//***************************************************
procedure TKolSystemMenu.Loaded;
begin
  fOldOnMessage := Application.OnMessage;
  Application.OnMessage := AppMessage;
	inherited Loaded;
end;


end.
